/* See accompanying file, LICENSE, for license information */

if(typeof BUBBAMON === 'undefined') {
    var BUBBAMON = {};
}

BUBBAMON.BubbaServer = function() {
   this.addr = BUBBAMON.prefs.address + "/web-admin/";
   this.generalData= {};
   this.timer = null;
   this.timeout_value = BUBBAMON.prefs.interval * 2;
   this.timeout_timer = null;
   this.enabled = false;
}

BUBBAMON.BubbaServer.prototype.start_timeout = function(observer) {
   this.timeout_timer = window.setTimeout(function(){ observer.notify(); }, this.timeout_value);
}

BUBBAMON.BubbaServer.prototype.stop_timeout = function() {
   // Stop timeout timer
   if(this.timeout_timer != null) {
      clearTimeout(this.timeout_timer);
      this.timeout_timer = null;
   }
}

BUBBAMON.BubbaServer.prototype.logOn = function(msg) {
   this.enabled = false;

   if(msg == undefined) {
      msg = "Connecting...";
   }

   BUBBAMON.setStatus(msg);

   var observer = new BUBBAMON.BubbaServer.LogonRetry(this);
   this.start_timeout(observer);

   BUBBAMON.HtmlConnection(this.addr + "index.php", "submit=yes" +
      "&uid=" + BUBBAMON.prefs.user +
      "&passwd=" + BUBBAMON.prefs.passwd, this.verifyLogOn, this, BUBBAMON.prefs.interval);
}



BUBBAMON.BubbaServer.prototype.logOff = function() {
   this.stop_timeout();
   this.enabled = false;
   if(this.timer != null) {
      clearTimeout(this.timer);
      this.timer = null;
   }

   BUBBAMON.HtmlConnection(this.addr + "index.php", "logout=yes", null, null, BUBBAMON.prefs.interval);
}

BUBBAMON.BubbaServer.prototype.pause = function() {
   this.enabled = false;
   this.stop_timeout();
   if(this.timer != null) {
      clearTimeout(this.timer);
      this.timer = null;
   }
}

BUBBAMON.BubbaServer.prototype.verifyLogOn = function(response, origin) {
   // Test we are logged in.
   if(response == "") {
      return;
   }

   origin.stop_timeout();

   if(response.search(/Invalid user/) == -1) {
      BUBBAMON.setStatus("Connected");
   	origin.refresh();
   } else {
   	if (BUBBAMON.prefs.nagCounter == 0) {
         BUBBAMON.setStatus("Invalid user/passwd");
         alert("Invalid user/password combination.\n" + 
               "Verify you are using the right identity.");
         // Annoying nag counter.
         BUBBAMON.prefs.nagCounter++;			
      }
   }
}
   
BUBBAMON.BubbaServer.prototype.refresh = function() {
   this.enabled = true;
   if(this.timer != null) {
      clearTimeout(this.timer);
      this.timer = null;
   }
   BUBBAMON.HtmlConnection(this.addr + "dllist.php", null, this.parseDLList, this, BUBBAMON.prefs.interval);

   this.hddtemp = null;
   this.hddname = "sda";
   this.generalData = null;

   BUBBAMON.HtmlConnection(this.addr + "index.php", null, this.parseGeneral, this, BUBBAMON.prefs.interval);
   if(BUBBAMON.prefs.hddtemp != "") {
      BUBBAMON.HtmlConnection(BUBBAMON.prefs.address + BUBBAMON.prefs.hddtemp, null, this.parseTemp, this, BUBBAMON.prefs.interval);
   }

   var observer = new BUBBAMON.BubbaServer.RefreshRetry(this);
   this.start_timeout(observer);
}

BUBBAMON.BubbaServer.prototype.formatLink = function(str, p1, offset, s)
{
   var result = 'value="' + encodeURIComponent(p1) + '"';
   dump(str + " => " + result + "\n");
   return result;

}

BUBBAMON.BubbaServer.prototype.parseDLList = function(html, origin) {
   if(!origin.enabled) {
      return;
   }

   // Workaround for weird bug when downloading eg. security updates
   // from apple (the link contain '&' and that seems to confuse the DOM parser
   //  We're looking for value fields with a link.
   var regxp = /value=\"(http:\/\/.*)\"/g;
   html = html.replace(regxp, origin.formatLink);

   var doc = (new DOMParser()).parseFromString(html, "text/xml");
   var tables = doc.getElementsByTagName("table");

   var downloads = { "dls": {} };

   for(var a = 0; a < tables.length; a++)
   {
      var dl = new BUBBAMON.BubbaServer.Download(tables[a], a);
      downloads.dls[a] = dl;

      if(BUBBAMON.prefs.autoDelete) {
         var srcPath = unescape(dl.source);
         var srcPattern = unescape(BUBBAMON.prefs.intermediateSrc);
         if(0 == srcPath.search(srcPattern)) {
            // Delete locally source torrent file
            var file = srcPath.substr(srcPattern.length);
            BUBBAMON.removeLocalTorrent(file);
         }
      }
   }

   // Update list in popup menu
   BUBBAMON.updateDownloads(downloads);
   
   // Start a timer for later update
   origin.timer = window.setTimeout(function(){ origin.refresh(); }, BUBBAMON.prefs.interval);
}

BUBBAMON.BubbaServer.prototype.parseTemp = function(html, origin) {
   if(!origin.enabled) {
      return;
   }
   var fields = html.split("|");
   var value = fields[3];
   var unit = fields[4];
   origin.hddtemp = value + "°" + unit;
   origin.hddname = fields[1].split("/")[2];

   // Test if uptime has arrived. If so, update everything
   if(origin.generalData) {
      origin.generalData.disks[origin.hddname] = { "temp": origin.hddtemp };
      BUBBAMON.updateGenerals(origin.generalData);
   }
}

BUBBAMON.BubbaServer.prototype.parseGeneral = function(html, origin) {
   if(!origin.enabled) {
      return;
   }

   origin.stop_timeout();

   var doc = (new DOMParser()).parseFromString(html, "text/xml");
   var a;
   var tables = doc.getElementsByTagName("table");
   origin.generalData = {};

   if(tables.length == 3) {
      var data = tables[1].getElementsByTagName("td");
      var divDate = doc.getElementById("date");
      var text = divDate.childNodes[4].data;

      var re = /Ver:\s*([^<]*).*/;
      origin.generalData.version =text.replace(re, "$1");
      var u = data[10].childNodes[0].data;
      if(u.search(/days/) != -1) {
         var re = /\s*([0-9]*)\s*days (\d*):(\d*):(\d*)/;
         origin.generalData.uptime = [ 
            parseInt(u.replace(re, "$1"), 10), 
            parseInt(u.replace(re, "$2"), 10), 
            parseInt(u.replace(re, "$3"), 10),
            parseInt(u.replace(re, "$4"), 10)
         ];
      } else {
         var re = /\s*(\d*):(\d*):(\d*)/;
         origin.generalData.uptime = [ 
            0,
            parseInt(u.replace(re, "$1"), 10), 
            parseInt(u.replace(re, "$2"), 10), 
            parseInt(u.replace(re, "$3"), 10)
         ];
      }
      origin.generalData.partitions = {
         "system": {
            "size": BUBBAMON.BubbaServer.translate(data[1].childNodes[0].data),
            "free": BUBBAMON.BubbaServer.translate(data[3].childNodes[0].data)
         }
      };
      origin.generalData.disks = {};

      if((BUBBAMON.prefs.hddtemp == "") || (origin.hddtemp != null)) {
         origin.generalData.disks[origin.hddname] = { "temp": origin.hddtemp };
         BUBBAMON.updateGenerals(origin.generalData);
      }
   } else {
      origin.enabled = false;
      origin.logOn("Reconnecting...");
   }
}

BUBBAMON.BubbaServer.prototype.addDownload = function(url, uuid) {
   this.pause();

   var observer = new BUBBAMON.BubbaServer.Resume(this);
   this.start_timeout(observer);

   BUBBAMON.HtmlConnection(this.addr + "downloads.php", "add_download=Add" + 
         "&uuid=" + uuid + 
         "&url=" + url, this.restart, this, BUBBAMON.prefs.interval);
}

BUBBAMON.BubbaServer.prototype.cancelDownload = function(src, uuid) {
   this.pause();

   var observer = new BUBBAMON.BubbaServer.Resume(this);
   this.start_timeout(observer);

   BUBBAMON.HtmlConnection(this.addr + "downloads.php", "do=Cancel" +
         "&remove=" + src + 
         "&uuid=" + uuid, this.restart, this, BUBBAMON.prefs.interval);
}

BUBBAMON.BubbaServer.prototype.restart = function(response, origin) {
   origin.stop_timeout();
   origin.refresh();
}

BUBBAMON.BubbaServer.Resume = function(origin)
{
   this.origin = origin;
   this.notify = function() {
      this.origin.stop_timeout();
      this.origin.refresh();
   }
}

BUBBAMON.BubbaServer.LogonRetry = function(origin)
{
   this.origin = origin;
   this.notify = function() {
      if(this.origin.enabled) {
         this.origin.logOn("Retry logon...");
      }
   }
}

BUBBAMON.BubbaServer.RefreshRetry = function(origin)
{
   this.origin = origin;
   this.notify = function() {
      this.origin.refresh();
   }
}

BUBBAMON.BubbaServer.translate = function(txt) {
   var value = txt.replace(/[^0-9\.]/g, "");
   var unit = txt.replace(/[ 0-9\.\/s]/g, "");
   switch(unit) {
      case "K":
      case "KB": value *= 1024; break;
      case "M":
      case "MB": value *= (1024*1024); break;
      case "G":
      case "GB": value *= (1024*1024*1024); break;
      case "T":
      case "TB": value *= (1024*1024*1024*1024); break;
      default: value *= 1; break;
   }
   return value;
}

BUBBAMON.BubbaServer.Download = function(table) {
   var td = table.getElementsByTagName("td");
   this.name = td[0].childNodes[0].childNodes[0].data;
   var progress = td[1].childNodes[0].childNodes[0].data;

   var done;

   this.status = 0;
   this.size = 0;
   this.downloaded = 0;

   // Examine index 3. A small-element indicates a torrent download.
   if(td[3] && td[3].childNodes[0] && (td[3].childNodes[0].nodeType == 1) && 
         td[3].childNodes[0].tagName.toLowerCase() == "small") {
      this.info = {
         "type": "torrent",
         "state": "no info",
         "download_rate": 0,
         "total_download": 0,
         "upload_rate": 0,
         "total_upload": 0,
         "num_peers": 0,
         "num_seeds": 0
      };
      var txtInfo = td[3].childNodes[0].childNodes[0].childNodes[0].data;
      var xferstats = "DL: 0B/s (0 B) UL: 0B/s (0 B)";
      var connStats = " (0 seeds 0 peers)";

      if(txtInfo.search("Downloading") != -1 ||
            txtInfo.search("Seeding") != -1)
      {
         xferstats = td[3].childNodes[0].childNodes[3].data;
         connStats = td[3].childNodes[0].childNodes[1].data;
      }
      var expr ="DL:\\s*([^/]*)\\/s\\s*\\(([^)]*)\\)\\s*UL:\\s*([^/]*)\\/s\\s*\\(([^)]*)\\)";
      var re = new RegExp(expr);
      //var re = /DL:\s*([^/]*)\/s\s*\(([^)]*)\)\s*UL:\s*([^/]*)\/s\s*\(([^)]*)\)/; 
      this.info.download_rate = 
         BUBBAMON.BubbaServer.translate(xferstats.replace(re, "$1"));
      this.info.total_download = 
         BUBBAMON.BubbaServer.translate(xferstats.replace(re, "$2"));
      this.info.upload_rate =
         BUBBAMON.BubbaServer.translate(xferstats.replace(re, "$3"));
      this.info.total_upload =
         BUBBAMON.BubbaServer.translate(xferstats.replace(re, "$4"));

      re = /\(([0-9]+) seeds ([0-9]+) peers\)/;
      this.info.num_seeds = connStats.replace(re, "$1");
      this.info.num_peers = connStats.replace(re, "$2");

      if (txtInfo.search("Downloading") != -1) {
         this.info.state = "Downloading";
         this.status = this.STATE_IN_PROGRESS;
         done = progress.replace(/(.*)%/, "$1");
         // Prevent not completed downloads from reporting 100%
         if(done == 100) {
            done = 99;
         }
      }
      else if(txtInfo.search("Connecting") != -1) {
         done = 0;
         this.info.state = "Connecting";
         this.status = this.STATE_NOT_STARTED;
      }
      else if(txtInfo.search("Seeding") != -1) {
         done = 100;
         this.info.state = "Seeding";
         this.status = this.STATE_DOWNLOADED;
      }
      else if(txtInfo.search("Queued") != -1) {
         done = 0;
         this.info.state = "Queued";
         this.status = this.STATE_QUEUED;
      } 
      else if(txtInfo.search("Checking") != -1) {
         done = 0;
         this.info.state = "Checking";
         this.status = this.STATE_QUEUED;
      }
      else if(txtInfo.search("Allocating") != -1) {
         done = 0;
         this.info.state = "Allocating";
         this.status = this.STATE_QUEUED;
      }
      else if(txtInfo.search("Unknown") != -1) {
         done = 0;
         this.info.state = "Unknown";
         this.status = this.STATE_FAILED;
      }
      else if(txtInfo.search("Download failed") != -1) {
         done = 0;
         this.info.state = "Failed";
         this.status = this.STATE_FAILED;
      }
      else if(txtInfo.search("No info") != -1) {
         done = 0;
         this.info.state = "No info";
         this.status = this.STATE_FAILED;
      }
      else {
         // Oops, unknown info text
         done = 0;
         this.info.state = "Unknown";
         this.status = this.STATE_FAILED;
      }
   } else {
      this.info = {
         "type": "curl"
      };
      try {
         this.info.speed = BUBBAMON.BubbaServer.translate(td[3].childNodes[0].data);
         this.status = this.STATE_IN_PROGRESS;
         this.info.state = "Downloading";
      }
      catch(e) {
         this.info.speed = 0;
      }      

      switch(progress) {
         case "Complete":
            done = 100;
            this.info.state = "Done";
            this.status = this.STATE_DOWNLOADED;
            break;

         case "Failed":
            done = 0;
            this.info.state = "Failed";
            this.status = this.STATE_FAILED;
            break;

         case "Canceled":
            done = 0;
            this.info.state = "Canceled";
            this.status = this.STATE_CANCEL_DONE;
            break;

         default:
            done = progress.replace(/(.*)%/, "$1");
            // Prevent not completed downloads from reporting 100%
            if(done == 100) {
               done = 99;
            }
            break;
      }
   }
   try {
      this.size = BUBBAMON.BubbaServer.translate(td[4].childNodes[0].data.replace(/Size: /, ""));
      this.downloaded = (done / 100) * this.size;
   }
   catch(e) {
      this.size = 0;
      this.downloaded = 0;
   }

   var inp = table.getElementsByTagName("input");
   this.source = inp[0].getAttribute("value");
   this.uuid = inp[1].getAttribute("value");
}
BUBBAMON.BubbaServer.Download.prototype.STATE_QUEUED = 1;
BUBBAMON.BubbaServer.Download.prototype.STATE_DOWNLOADED = 2;
BUBBAMON.BubbaServer.Download.prototype.STATE_CANCEL_IN_PROGRESS = 3;
BUBBAMON.BubbaServer.Download.prototype.STATE_CANCEL_DONE = 4;
BUBBAMON.BubbaServer.Download.prototype.STATE_IN_PROGRESS = 5;
BUBBAMON.BubbaServer.Download.prototype.STATE_FAILED = 6;


