/*
 * Copyright (C) 2009 Johan Stenarson
 * See accompanying file, LICENSE, for license information 
 */

if(typeof BUBBAMON === 'undefined') {
    var BUBBAMON = {}
}

BUBBAMON.fillMenulist = function(id) {
    var locale = new BUBBAMON.StringBundle("BubbaMonPrefs");

    var element = document.getElementById(id);
    while(element.firstChild) {
        element.removeChild(element.firstChild);
    }

    var popup = document.createElement("menupopup");
    popup.setAttribute("orient", "vertical");

    popup.appendChild(BUBBAMON.createItem("{name}"));
    popup.appendChild(BUBBAMON.createItem(locale.getFormattedText('bubbamon.opt.combo.download.size', [ "{size}" ])));
    popup.appendChild(BUBBAMON.createItem(locale.getFormattedText('bubbamon.opt.combo.download.downloaded', ["{downloaded}"])));
    popup.appendChild(BUBBAMON.createItem(locale.getFormattedText('bubbamon.opt.combo.download.uploaded', ["{uploaded}"])));
    popup.appendChild(BUBBAMON.createItem(locale.getFormattedText('bubbamon.opt.combo.download.status', ["{status}"])));
    popup.appendChild(BUBBAMON.createItem(locale.getFormattedText('bubbamon.opt.combo.download.seeds', ["{seeds}"])));
    popup.appendChild(BUBBAMON.createItem(locale.getFormattedText('bubbamon.opt.combo.download.peers', ["{peers}"])));
    popup.appendChild(BUBBAMON.createItem(locale.getFormattedText('bubbamon.opt.combo.download.down', ["{down}"])));
    popup.appendChild(BUBBAMON.createItem(locale.getFormattedText('bubbamon.opt.combo.download.up', ["{up}"])));
    popup.appendChild(BUBBAMON.createItem(locale.getFormattedText('bubbamon.opt.combo.download.ratio', ["{ratio}"])));

    element.appendChild(popup);
}

BUBBAMON.createItem = function(name) {
    var element = document.createElement("menuitem");
    element.setAttribute("label", name);
    return element;
}

BUBBAMON.loadPreferences = function() {
	const prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
	const bmBranch = prefService.getBranch("extensions.bubbamon.");

	// General
	document.getElementById("BM_TXT_address").value = bmBranch.getCharPref("address");

	const user = bmBranch.getCharPref("user");
	document.getElementById("BM_TXT_name").value = user;
    if(user !== "") {
    	var pwdMan = new BUBBAMON.PasswordManager(user);
    	document.getElementById("BM_TXT_password").value = pwdMan.get();
    }
	document.getElementById("BM_TXT_interval").value = bmBranch.getIntPref("interval");

	// Download modes
	document.getElementById("BM_CHK_immediate").checked = bmBranch.getIntPref("downloadmode") & 1;
	document.getElementById("BM_CHK_intermediate").checked = bmBranch.getIntPref("downloadmode") & 2;
	document.getElementById("BM_TXT_directory").value = bmBranch.getCharPref("intermediateDst");
	document.getElementById("BM_TXT_uri").value = bmBranch.getCharPref("intermediateSrc");
	document.getElementById("BM_CHK_delete").checked = bmBranch.getBoolPref("autodelete");

	// Appearance
	document.getElementById("BM_LST_ongoing_left").label = bmBranch.getCharPref("ongoing_left");
	document.getElementById("BM_LST_ongoing_upper").label = bmBranch.getCharPref("ongoing_upper");
	document.getElementById("BM_LST_ongoing_lower").label = bmBranch.getCharPref("ongoing_lower");
	document.getElementById("BM_LST_completed_left").label = bmBranch.getCharPref("completed_left");
	document.getElementById("BM_LST_completed_upper").label = bmBranch.getCharPref("completed_upper");
	document.getElementById("BM_LST_completed_lower").label = bmBranch.getCharPref("completed_lower");

	// BUBBA|server
	var addr = bmBranch.getCharPref("hddtemp");
	document.getElementById("BM_TXT_hddtemp").value = addr;
	document.getElementById("BM_CHK_hddtemp").checked = addr != "";

	// Update GUI so it show the actual settings
	BUBBAMON.toggleIntermediateSettings(0 == (bmBranch.getIntPref("downloadmode") & 2));
	BUBBAMON.toggleHDDtempSettings(addr == "");
	// document.getElementById("BM_TAB_bubbaServer").hidden = bmBranch.getIntPref("serverType") != 1;
	BUBBAMON.updateProgressbar(document.getElementById("BM_LST_ongoing_left"));
	BUBBAMON.updateProgressbar(document.getElementById("BM_LST_ongoing_upper"));
	BUBBAMON.updateProgressbar(document.getElementById("BM_LST_ongoing_lower"));
	BUBBAMON.updateProgressbar(document.getElementById("BM_LST_completed_left"));
	BUBBAMON.updateProgressbar(document.getElementById("BM_LST_completed_upper"));
	BUBBAMON.updateProgressbar(document.getElementById("BM_LST_completed_lower"));
	// Set correct version
    try {
        var version;
        var oEM = Components.classes["@mozilla.org/extensions/manager;1"].
            getService(Components.interfaces.nsIExtensionManager);
        if (oEM.getItemForID) {
            version = oEM.getItemForID("bubbamon@thevoid.webhop.info").version;
        } else {
            version = oEM.getItemList("bubbamon@thevoid.webhop.info", null, {})[0].version;
        }
        document.getElementById("BM_LBL_version").value = "version " + version;
    }
    catch (e) {
        Components.utils.import("resource://gre/modules/AddonManager.jsm");
        AddonManager.getAddonByID("bubbamon@thevoid.webhop.info", function(addon) {
            BUBBAMON.version = addon.version;
            document.getElementById("BM_LBL_version").value = "BubbaMon v" + addon.version;
        });
    }
	
}

BUBBAMON.initPreferences = function() {
    BUBBAMON.fillMenulist("BM_LST_ongoing_left");
    BUBBAMON.fillMenulist("BM_LST_ongoing_upper");
    BUBBAMON.fillMenulist("BM_LST_ongoing_lower");
    BUBBAMON.fillMenulist("BM_LST_completed_left");
    BUBBAMON.fillMenulist("BM_LST_completed_upper");
    BUBBAMON.fillMenulist("BM_LST_completed_lower");

    BUBBAMON.loadPreferences();
}

BUBBAMON.savePreferences = function() {
    const prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    const bmBranch = prefService.getBranch("extensions.bubbamon.");

    // General
    var addr = document.getElementById("BM_TXT_address").value;
    var user = document.getElementById("BM_TXT_name").value;
	// Password is stored by password manager (for privacy)
	// We need to trigger a preference event on the fake setting so main
	// application knows it has changed.
    var pwdMan = new BUBBAMON.PasswordManager(user);
    if(document.getElementById("BM_TXT_password").value !==  pwdMan.get()) {
        pwdMan.set(document.getElementById("BM_TXT_password").value);
        bmBranch.setBoolPref("password", !bmBranch.getBoolPref("password"));
    }
    bmBranch.setCharPref("address", addr);
    bmBranch.setCharPref("user", user);
    bmBranch.setIntPref("interval", document.getElementById("BM_TXT_interval").value);

    // Download modes
    var mode = 0;
    mode += document.getElementById("BM_CHK_immediate").checked ? 1 : 0;
    mode += document.getElementById("BM_CHK_intermediate").checked ? 2 : 0;
    bmBranch.setIntPref("downloadmode",  mode);
    bmBranch.setCharPref("intermediateDst", document.getElementById("BM_TXT_directory").value);
    bmBranch.setCharPref("intermediateSrc", document.getElementById("BM_TXT_uri").value);
    bmBranch.setBoolPref("autodelete", document.getElementById("BM_CHK_delete").checked);

    // Appearance
    bmBranch.setCharPref("ongoing_left", document.getElementById("BM_LST_ongoing_left").label);
    bmBranch.setCharPref("ongoing_upper", document.getElementById("BM_LST_ongoing_upper").label);
    bmBranch.setCharPref("ongoing_lower", document.getElementById("BM_LST_ongoing_lower").label);
    bmBranch.setCharPref("completed_left", document.getElementById("BM_LST_completed_left").label);
    bmBranch.setCharPref("completed_upper", document.getElementById("BM_LST_completed_upper").label);
    bmBranch.setCharPref("completed_lower", document.getElementById("BM_LST_completed_lower").label);

    // BUBBA|server
    if(document.getElementById("BM_CHK_hddtemp").checked) {
        bmBranch.setCharPref("hddtemp", document.getElementById("BM_TXT_hddtemp").value);
    } else {
        bmBranch.setCharPref("hddtemp", "");
    }
}

BUBBAMON.toggleHDDtempSettings = function(state) {
    document.getElementById("BM_TXT_hddtemp").disabled = state;
    document.getElementById("BM_BTN_hddtemp").disabled = state;
}

BUBBAMON.toggleIntermediateSettings = function(state) {
    document.getElementById("BM_TXT_directory").disabled = state;
    document.getElementById("BM_BTN_browse").disabled = state;
    document.getElementById("BM_TXT_uri").disabled = state;
    document.getElementById("BM_CHK_delete").disabled = state;
    document.getElementById("BM_BTN_verify").disabled = state;
}

BUBBAMON.testIntermediate = function() {
    var writeOk = false;
    var locale = new BUBBAMON.StringBundle("BubbaMonPrefs");
    var data = "If none of us believe in war, then can you tell me what the weapon's for?\n" +
               "Listen to me everyone, if the button is pushed there'll be nowhere to run! // Killer Of Giants - Ozzy Osbourne";
    try {
        var path = document.getElementById("BM_TXT_directory").value;
        if(!path.match(/\/$/)) {
            path= path+ "/";
        }
        path = path + "killer_of_giant.txt";
        BUBBAMON.writeAFile(path, data);
        writeOk = true;
    } 
    catch(ex) {
        alert(locale.getText("bubbamon.dlg.cant_write"));
    }

    if(writeOk) {
        try {
            // Try reading it via webserver
            var addr = document.getElementById("BM_TXT_uri").value;
            if(!addr.match(/\/$/)) {
                addr = addr + "/";
            }
            addr = addr + "killer_of_giant.txt";
            BUBBAMON.HtmlConnection(addr, null, BUBBAMON.verifyDownload, data, 5000);
        }
        catch(ex) {
            alert(locale.getText("bubbamon.dlg.cant_download", [ ex.message ]));
        }
    }
}

BUBBAMON.verifyDownload = function(html, data) {
    var locale = new BUBBAMON.StringBundle("BubbaMonPrefs");

    if(html === data) {
        var path = BUBBAMON.getLocalFile(document.getElementById("BM_TXT_directory").value);
        if( !path ) { return false; }

        try {
            path.append("killer_of_giant.txt");
            if(path.isFile()) {
                path.remove(false);
            }
            alert(locale.getText("bubbamon.dlg.intermediate_ok"));
        }
        catch(ex) {
            alert(locale.getFormattedText("bubbamon.dlg.cant_delete", [ ex.message ]));
        }
    } else {
        alert(locale.getText("bubbamon.dlg.cant_download"));
    }
}

BUBBAMON.writeAFile = function(name, data) {
    var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
        .createInstance(Components.interfaces.nsIFileOutputStream);
    var nsILocalFile = Components.interfaces.nsILocalFile;
    var file = Components.classes["@mozilla.org/file/local;1"]
        .createInstance(nsILocalFile);
    file.initWithPath(name);

    // use 0x02 | 0x10 to open file for appending.
    foStream.init(file, -1, 0666, 0);
    foStream.write(data, data.length);
    foStream.close();
}

BUBBAMON.getLocalFile = function(file) {
    const nsILocalFile = Components.interfaces.nsILocalFile;

    var dir = Components.classes["@mozilla.org/file/local;1"]
        .createInstance(nsILocalFile);

    try {
        dir.initWithPath(file);
        if(!dir.isDirectory()) {
            throw("Not a directory");
        }
    }
    catch(ex) {
        return false;
    }

    return dir;
}

BUBBAMON.testConnection = function() {
    var addr = document.getElementById("BM_TXT_address").value;

    if(!addr.match(/^http:/)) {
        addr = "http://" + addr;
    }

    var ioService = Components.classes["@mozilla.org/network/io-service;1"]
        .getService(Components.interfaces.nsIIOService);
    var uri = ioService.newURI(addr,null,null);
    addr = uri.prePath;
    document.getElementById("BM_TXT_address").value = uri.prePath;

    BUBBAMON.HtmlConnection(addr + "/web-admin/", null, BUBBAMON.verifyFirst, addr, 5000);
}

BUBBAMON.verifyFirst = function(html, addr)
{
    var locale = new BUBBAMON.StringBundle("BubbaMonPrefs");
    if(html.match("Please enter username and password to log in")) {
        alert(locale.getText("bubbamon.dlg.found_b1"));
    } else {
        BUBBAMON.HtmlConnection(addr + "/admin/", null, BUBBAMON.verifySecond, null, 5000);
    }
}

BUBBAMON.verifySecond = function(html, addr)
{
    var locale = new BUBBAMON.StringBundle("BubbaMonPrefs");
    if(html.match("Bubba|TWO - Login")) {
        alert(locale.getText("bubbamon.dlg.found_b2"));
    } else if(html.match("B3 - Home")) {
        alert(locale.getText("bubbamon.dlg.found_b3"));
    } else {
        alert(locale.getText("bubbamon.dlg.no_bubba"));
    }
}

BUBBAMON.testTemperature = function() {
    var addr = document.getElementById("BM_TXT_address").value + document.getElementById("BM_TXT_hddtemp").value;
    
    BUBBAMON.HtmlConnection(addr, null, BUBBAMON.verifyTemperature, null, 5000);
}

BUBBAMON.verifyTemperature = function(response, origin)
{
    var locale = new BUBBAMON.StringBundle("BubbaMonPrefs");
    var fields = response.split("|");
    var message;

    if((fields.length == 6) && fields[1].match("^\/dev\/")) {
        message = locale.getText("bubbamon.dlg.hddtemp_found");
    } else if(response !== "") {
        message = locale.getText("bubbamon.dlg.hddtemp_not_found");
    } else {
        message = locale.getText("bubbamon.dlg.hddtemp_no_server");
    }
    alert(message);
}

BUBBAMON.updateProgressbar = function(source) {
    var locale = new BUBBAMON.StringBundle("BubbaMonPrefs");
    var fields = /BM_LST_([^_]*)_([^_]*)/.exec(source.id);

    var result = source.label;
    result = result.replace("{name}", "File_being_downloaded.bin");
    result = result.replace("{size}", "123.45MB");
    result = result.replace("{downloaded}", "42.4MB");
    result = result.replace("{uploaded}", "24.8MB");
    result = result.replace("{status}", locale.getText("bubbamon.status.downloading"));
    result = result.replace("{seeds}", "42");
    result = result.replace("{peers}", "24");
    result = result.replace("{down}", "123.4kB/s");
    result = result.replace("{up}", "43.2kB/s");
    result = result.replace("{ratio}", "1.04");

    document.getElementById("BM_LBL_" + fields[1] + "_" + fields[2]).value = result;
}

BUBBAMON.browseLocalFolders = function() {
    var locale = new BUBBAMON.StringBundle("BubbaMonPrefs");
    const nsIFilePicker = Components.interfaces.nsIFilePicker;
    const nsILocalFile = Components.interfaces.nsILocalFile;

    var fp = Components.classes["@mozilla.org/filepicker;1"]
        .createInstance(nsIFilePicker);
    var displayDir = Components.classes["@mozilla.org/file/local;1"]
        .createInstance(nsILocalFile);

    //If path is unrecognised just continue without specifying displayDirectory
    try {
        displayDir.initWithPath( document.getElementById("intermediateDst").value );
        if( displayDir.isDirectory() ) {
            fp.displayDirectory = displayDir;
        }
    }
    catch(ex) { }

    fp.init(window, locale.getText('bubbamon.dlg.select_folder'), nsIFilePicker.modeGetFolder);
    var ret = fp.show();

    if (ret == nsIFilePicker.returnOK) {
        document.getElementById("BM_TXT_directory").value = fp.file.path;
    }
}
